// Copyright 1999-2017. Parallels IP Holdings GmbH. All Rights Reserved.

Jsw.namespace('PleskExt.Git');

PleskExt.Git.PullWatcher = Class.create({

    loadPullTasks: function() {
        if (this._running) {
            return;
        }
        this._running = true;
        this._loadPullTasks();
    },

    _loadPullTasks: function() {
        var context = this;
        new Ajax.Request(PleskExt.Git.Config.listPullTasksUrl, {
            method: 'get',
            onSuccess: function(response) {
                var data = response.responseText.evalJSON();
                context._processFailedTasks(data.failedTasks);
                context._processRepoIds(data.repoIds);
            },
            on400: function() {
                context._running = false;
                // prevents page reload
                return false;
            }
        });
    },

    _processRepoIds: function(repoIds) {
        if (!repoIds) {
            this._running = false;
            $$('.active-list-item a[href^=javascript\\:PleskExt\\.Git\\.actions\\.pull]').each(
                function(button) {
                    this.removeSpinner(button);
                }, this);
            return;
        }
        $$('.active-list-item a[href^=javascript\\:PleskExt\\.Git\\.actions\\.pull]').each(function(button) {
            var href = button.getAttribute('href');
            var id = href.replace('javascript:PleskExt.Git.actions.pull(', '').replace(');', '');
            if (repoIds.indexOf(id) === -1) {
                this.removeSpinner(button);
            } else {
                this.setSpinner(button);
            }
        }, this);
        var context = this;
        window.setTimeout(function() {
            context._loadPullTasks();
        }, 5000);
    },

    _processFailedTasks: function(failedTasks) {
        if (!failedTasks) {
            return;
        }
        for (var id in failedTasks) {
            if (!failedTasks.hasOwnProperty(id)) {
                continue;
            }
            var taskId = id;
            var messageId = 'js-git-pull-task-id-' + id;
            if ($(messageId)) {
                continue;
            }
            Jsw.addStatusMessage('error', failedTasks[id], {
                id: messageId,closable: true,
                onClose: this._onCloseErrorMessage.bind(this, taskId, messageId)
            });
        }
    },

    _onCloseErrorMessage: function(taskId, messageId) {
        $(messageId).remove();
        new Ajax.Request(Jsw.prepareUrl('/task/task-remove/'), {
            method: 'post',
            parameters: {'ids[]': [taskId]},
            on400: function() {
                // prevents page reload
                return false;
            }
        });
    },

    setSpinner: function(button) {
        button.addClassName('disabled');
        var spinner = button.down('.icon-indicator');
        if (!spinner) {
            button.insert({top: '<i class="icon-indicator"></i>'});
        }
    },

    removeSpinner: function(button) {
        var spinner = button.down('.icon-indicator');
        if (spinner) {
            spinner.remove();
        }
        button.removeClassName('disabled');
    }
});

PleskExt.Git.pullWatcher = new PleskExt.Git.PullWatcher();

Jsw.namespace('PleskExt.Git.actions');

PleskExt.Git.actions.pull = function (id) {
    var button = $$('.active-list-item a[href=javascript\\:PleskExt\\.Git\\.actions\\.pull\\(' + id + '\\)\\;]').first();
    if (!button) {
        return;
    }
    PleskExt.Git.pullWatcher.setSpinner(button);

    new Ajax.Request(PleskExt.Git.Config.pullUrl, {
        method: 'post',
        parameters: {
            id: id
        },
        onSuccess: function (response) {
            var data = response.responseText.evalJSON();
            if (data.status === 'success') {
                PleskExt.Git.pullWatcher.loadPullTasks();
            } else {
                PleskExt.Git.pullWatcher.removeSpinner(button);
                new Jsw.Popup({
                    title: data.title,
                    content: data.message,
                    closeButtonEnabled: true
                });
            }

        }.bind(this)
    });
};

PleskExt.Git.actions.deploy = function (id) {
    new Ajax.Request(PleskExt.Git.Config.deployUrl, {
        method: 'post',
        parameters: {
            id: id
        },
        onSuccess: function (response) {
            var data = response.responseText.evalJSON();
            new Jsw.Popup({
                title: data.title,
                content: data.message,
                closeButtonEnabled: true
            });
        }.bind(this)
    });
};
